// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

#ifndef CHROMADB_H
#define CHROMADB_H

#include <QObject>
#include <QRestAccessManager>
#include "qtaiapiexports_p.h"

class QTAIAPI_EXPORT ChromaDb : public QObject
{
    Q_OBJECT
    Q_PROPERTY(bool connected READ connected WRITE connect NOTIFY connectedChanged FINAL)
    Q_PROPERTY(bool error READ error WRITE setError NOTIFY errorChanged FINAL)
public:
    explicit ChromaDb(QObject *parent = nullptr);

    bool connected() const;
    void connect(bool c);
    Q_INVOKABLE void reset();

    bool error() const;
    void setError(bool newError);

public Q_SLOTS:
    void storeEmbeddings(QVariantList embeddings, QVariantList documents);
    void fetchEmbeddings(QVariantList hints);

signals:
    void connectedChanged();
    void embeddingsStored();
    void embeddingsFound(QVariantList embeddings);

    void errorChanged();

private:
    void sendRequest(
        QString url,
        QJsonObject object,
        std::function<void(QJsonDocument, int)> lambda);
private:
    bool m_error {false};
    bool m_connected {false};
    QNetworkAccessManager m_manager;
    QRestAccessManager m_restApi;
    const QByteArray m_chromadb_url_base{"http://localhost:8000/api/v1/"};
    QByteArray m_chromadb_url_collection{};
};

#endif // CHROMADB_H
