#!/usr/bin/python3

# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

from http.server import BaseHTTPRequestHandler,HTTPServer
from os import curdir, sep
import torch
from diffusers import AutoPipelineForText2Image, LCMScheduler
import simplejson
import base64
from io import BytesIO
from PIL import Image



PORT_NUMBER = 8005

#This class will handles any incoming request from
#the browser 
class myHandler(BaseHTTPRequestHandler):
    model = ''


    #Handler for the POST requests
    def do_POST(self):
        print("do_POST");
        if self.path=="/send":
            self.data_string = self.rfile.read(int(self.headers['Content-Length']))
            print("data_string: " + self.data_string.decode())
            json_data = simplejson.loads(self.data_string)
            #print("json_data: " + simplejson.dumps(json_data))
            print("MODEL: " + json_data["model"])
            if self.model != json_data["model"]:
                self.pipeline = AutoPipelineForText2Image.from_pretrained(
                    json_data["model"],
                    #'black-forest-labs/FLUX.1-dev',
                    #'IDKiro/sdxs-512-dreamshaper',
                    #"stabilityai/stable-diffusion-2-1-base",
                    #'black-forest-labs/FLUX.1-schnell',
                    torch_dtype=torch.float32,
                    #variant="fp16",
                    use_safetensor=True).to('cpu')
                self.model = json_data["model"]

            image = self.pipeline(json_data["prompt"],
                height=512,
                width=512,
                guidance_scale=0.0,
                target_size=(1024, 1024),
                original_size=(4096, 4096),
                num_inference_steps=1
                #max_sequence_length=256
                ).images[0]
            
            buffered = BytesIO()
            image.save(buffered, format="PNG")
            b64image = base64.b64encode(buffered.getvalue())
            print("Sending response")
            self.send_response(200)
            self.end_headers()
            json_response = {}
            json_response["image"] = b64image
            self.wfile.write(simplejson.dumps(json_response).encode("utf-8"))
            return


try:
    #Create a web server and define the handler to manage the
    #incoming request
    server = HTTPServer(('', PORT_NUMBER), myHandler)
    print('Started httpserver on port ' , PORT_NUMBER)


    #Wait forever for incoming htto requests
    server.serve_forever()

except KeyboardInterrupt:
    print('^C received, shutting down the web server')
    server.socket.close()


