// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

#include "qaimodel.h"
#include "qasraimodel_p.h"
#include <QDir>
#include <QJsonDocument>
#include <QJsonObject>
#include <QNetworkReply>
#include <QRestReply>

QAsrAiModel::QAsrAiModel()
    : AiModelPrivateInterface(),
      m_manager(this)
    , m_restApi(&m_manager)
{
}

void QAsrAiModel::pushData(QVariantList data, int seed)
{
    qDebug() << "QAsrAiModel::pushData(): data:" << data;

    if (data.isEmpty() || data.first().toUrl().isEmpty()) {
        emit dataReceived(data.first().toUrl().toLocalFile());
        return;
    }

    QNetworkRequest request(QUrl("http://localhost:8003/send"));
    request.setRawHeader("Content-Type", "application/json");
    QJsonDocument doc;
    QJsonObject obj = doc.object();
    obj["model"] = m_owner->model();
    obj["file"] = data.first().toUrl().toLocalFile();
    //obj["stream"] = false;
    doc.setObject(obj);
    m_restApi.post(request, doc.toJson(), this, [this](QRestReply &reply) {
        if (auto json = reply.readJson()) {
            qDebug() << "[\"response\"]=" << json->object()["response"].toString();
            emit dataReceived(json->object()["response"].toString().toUtf8());
        }
    });
}
