// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

#include "qaimodel.h"
#include "qdiffuseraimodel_p.h"
#include <QJsonDocument>
#include <QJsonObject>
#include <QNetworkReply>
#include <QRestReply>
#include <QImage>

QDiffuserAiModel::QDiffuserAiModel()
    : AiModelPrivateInterface(),
      m_manager(this)
    , m_restApi(&m_manager)
{
}

void QDiffuserAiModel::pushData(QVariantList data, int seed)
{
    qDebug() << "QDiffuserAiModel::pushData(): data:" << data;

    if (data.isEmpty() || data.first().toString().isEmpty()) {
        emit dataReceived(data.first().toString());
        return;
    }

    QNetworkRequest request(QUrl("http://localhost:8005/send"));
    request.setRawHeader("Content-Type", "application/json");
    QJsonDocument doc;
    QJsonObject obj = doc.object();
    obj["model"] = m_owner->model();
    obj["prompt"] = data.first().toString();
    doc.setObject(obj);
    qDebug() << doc.toJson();

    m_restApi.post(request, doc.toJson(), this, [this](QRestReply &reply) {
        if (auto json = reply.readJson()) {
            emit dataReceived(QUrl(
                QString("data:image/png;base64,") + json->object()["image"].toString().toUtf8()));
        }
    });
}
