#include "qpiperaimodel_p.h"
#include "qaimodel.h"
#include <QJsonDocument>
#include <QJsonObject>
#include <QNetworkReply>
#include <QRestReply>
#include <QMediaDevices>

QPiperAiModel::QPiperAiModel()
    : AiModelPrivateInterface()
    , m_manager(this)
    , m_restApi(&m_manager)
{
    m_audioFormat.setSampleRate(22050);
    m_audioFormat.setChannelCount(1);
    m_audioFormat.setSampleFormat(QAudioFormat::Int16);

    m_audioSink.reset(new QAudioSink(QMediaDevices::defaultAudioOutput(),m_audioFormat));
    m_audioBuffer.reset(new QBuffer);
}

void QPiperAiModel::pushData(
    QVariantList data, int seed)
{
    qDebug() << "QPiperAiModel::pushData(): data:" << data;

    if (data.isEmpty() || data.first().toByteArray().isEmpty()) {
        emit dataReceived(data.first().toByteArray());
        return;
    }

    QNetworkRequest request(QUrl("http://localhost:5000/send"));
    request.setRawHeader("Content-Type", "application/json");
    QJsonDocument doc;
    QJsonObject obj = doc.object();
    obj["text"] = QString::fromUtf8(data.first().toByteArray());
    doc.setObject(obj);
    m_restApi.post(request, doc.toJson(), this, [this](QRestReply &reply) {
        if (auto json = reply.readJson()) {
            m_audioSink->reset();
            m_audioBuffer->close();
            m_audioBuffer->setData(QByteArray::fromBase64(json->object()["response"].toString().toUtf8()));
            m_audioBuffer->open(QIODevice::ReadOnly);

            // Start playback
            m_audioSink->start(m_audioBuffer.data());
        }
    });
    emit dataReceived(data.first().toByteArray());
}
