// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

#include <QTextToSpeech>
#include <QTimer>
#include "qtext2speechaimodel_p.h"

QText2SpeechAiModel::QText2SpeechAiModel()
    : AiModelPrivateInterface()
{
    m_speech.reset(new QTextToSpeech(this));
    m_speech->setRate(0.0);
    m_speech->setPitch(0.0);
    m_speech->setLocale(QLocale(QLocale::English, QLocale::UnitedKingdom));
    connect(m_speech.get(), &QTextToSpeech::stateChanged, [=](QTextToSpeech::State state) {
        if (state == QTextToSpeech::Ready)
            emit dataReceived("");
    });
}

void QText2SpeechAiModel::pushData(QVariantList data, int seed)
{
    m_speech->stop();

    if (!data.first().toByteArray().isEmpty()) {
        m_speech->say(QString::fromUtf8(data.first().toByteArray()));
    }
}
