// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

#include "qyoloaimodel.h"
#include "qaimodel.h"
#include <QJsonDocument>
#include <QJsonObject>
#include <QRestReply>
#include <QImage>
#include <QBuffer>
#include <QPixmap>
#include <QVideoFrame>

QYoloAiModel::QYoloAiModel()
    : AiModelPrivateInterface()
    , m_manager(this)
    , m_restApi(&m_manager)
{

}

void QYoloAiModel::pushData(QVariantList data, int seed)
{
    QImage image;
    if (data.first().canConvert<QImage>()) {
        image = data.first().value<QImage>();
        //image = imageTmp.convertToFormat(QImage::Format_RGB888).scaled(140, 80);
    }else if (data.first().canConvert<QVideoFrame>()) {
        image = data.first().value<QVideoFrame>().toImage();
        //image = imageTmp.convertToFormat(QImage::Format_RGB888).scaled(140, 80);
    } else {
        qDebug() << "error" << data;
        emit dataReceived("");
        return;
    }

    if (!image.bits()) {
        qDebug() << "error" << image;
        emit dataReceived("");
        return;
    }
    QBuffer bufferJpeg;
    bufferJpeg.open(QIODevice::WriteOnly);
    QPixmap pix = QPixmap::fromImage(image);
    pix.save(&bufferJpeg, "JPG");
    qDebug() << "QYoloAiModel::pushData(): data:" << image << "=> JPG:" << pix;

    QNetworkRequest request(QUrl("http://localhost:8004/send"));
    request.setRawHeader("Content-Type", "application/json");
    QJsonDocument doc;
    QJsonObject obj = doc.object();
    obj["model"] = m_owner->model();
    obj["frame"] = QString::fromLatin1(bufferJpeg.buffer().toBase64());
    doc.setObject(obj);
    m_restApi.post(request, doc.toJson(), this, [this](QRestReply &reply) {
        if (auto json = reply.readJson()) {
            emit dataReceived(json->toJson(QJsonDocument::Compact));
        }
        else
        {
            qDebug() << reply;
        }
    });
}
