// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

#ifndef QAIMODEL_H
#define QAIMODEL_H

#include <QObject>
#include <QQmlEngine>
#include "qaimodelinterface_p.h"

class AiModelPrivateInterface;

class QAiModel : public QObject
{
    Q_OBJECT
    QML_NAMED_ELEMENT(
        MultiModal)

    Q_PROPERTY(
        AiModelPrivateInterface::AiModelTypes type READ type WRITE setType NOTIFY typeChanged FINAL REQUIRED)
    Q_PROPERTY(
        QString prompt READ prompt WRITE setPrompt NOTIFY promptChanged FINAL)
    Q_PROPERTY(
        QString model READ model WRITE setModel NOTIFY modelChanged FINAL)
    Q_PROPERTY(
        QVariantList documents READ documents WRITE setDocuments NOTIFY documentsChanged FINAL)
    Q_PROPERTY(
        QVector<QAiModel*> inputs READ inputs WRITE setInputs NOTIFY inputsChanged FINAL)
    Q_PROPERTY(
        bool processing READ processing NOTIFY processingChanged FINAL)
    Q_PROPERTY(
        bool buffered READ buffered WRITE setBuffered NOTIFY bufferedChanged FINAL)
    Q_PROPERTY(
        bool optional READ optional WRITE setOptional NOTIFY optionalChanged FINAL)
    Q_PROPERTY(
        int seed READ seed WRITE setSeed NOTIFY seedChanged FINAL)

public:

    QAiModel();

    AiModelPrivateInterface::AiModelTypes type() const;
    void setType(const AiModelPrivateInterface::AiModelTypes &newType);

    bool processing() const;

    QString prompt() const { return m_prompt; }
    void setPrompt(const QString &newPrompt);

    QString model() const { return m_model; }
    void setModel(const QString &newModel);

    QVector<QAiModel*> inputs() const;
    void setInputs(QVector<QAiModel*>newInputs);

    QVariantList documents() const;
    void setDocuments(const QVariantList &newDocuments);

    Q_INVOKABLE void pushData(QVariant data);
    Q_INVOKABLE void clearBuffer();

signals:
    void typeChanged();

    void promptChanged();

    void modelChanged();

    void inputsChanged();

    void documentsChanged();

    void processingChanged();

    void gotResult(QVariant result);

    void bufferedChanged();

    void optionalChanged();

    void seedChanged();

private Q_SLOTS:
    void dataReceived(QVariant data);

public:
    AiModelPrivateInterface* interface() const { return m_interface.get(); }

    bool buffered() const;
    void setBuffered(bool newBuffered);

    bool optional() const;
    void setOptional(bool newOptional);

    int seed() const;
    void setSeed(int newSeed);

private:
    void processCombinedData(QVariant data);

private:
    QString m_model{};
    QString m_prompt{};
    QVector<QAiModel*> m_inputs;
    QString m_inputModel{};
    QAiModel* m_output{nullptr};
    QSharedPointer<AiModelPrivateInterface> m_interface;
    QVariantList m_documents;
    AiModelPrivateInterface::AiModelTypes m_type;
    QVariant m_buffer;  // TODO: replace with QVariant
    int m_seed{0};
    bool m_processing {false};
    bool m_buffered {false};
    bool m_optional {false};
};


#endif // QAIMODEL_H
