// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

#ifndef QAIMODELINTERFACE_P_H
#define QAIMODELINTERFACE_P_H

#include <QObject>
#include <QVariant>
#include "qtaiapiexports_p.h"

class QAiModel;

class QTAIAPI_EXPORT AiModelPrivateInterface : public QObject
{
    Q_OBJECT
public:
    enum AiModelType {
        InputText  = 0x00001,
        InputAudio = 0x00002,
        InputVideo = 0x00004,
        InputImage = 0x00008,
        InputJson  = 0x00010,

        OutputText  = 0x00100,
        OutputAudio = 0x00200,
        OutputVideo = 0x00400,
        OutputImage = 0x00800,
        OutputJson  = 0x01000,
    };
    Q_FLAG(AiModelType);
    Q_DECLARE_FLAGS(AiModelTypes, AiModelType)

    AiModelPrivateInterface() {}
    void init(QAiModel *owner) { m_owner = owner;}
    virtual ~AiModelPrivateInterface() {}

    virtual void pushData(QVariantList data, int seed) = 0;
    virtual void setRag(QVariantList data) {}


Q_SIGNALS:
    void dataReceived(QVariant data);
    void processingChanged();
    void embeddingsReceived(QVariantList embeddings, QVariantList documents);

public:
    QAiModel *m_owner;
};
Q_DECLARE_OPERATORS_FOR_FLAGS(AiModelPrivateInterface::AiModelTypes)

class QTAIAPI_EXPORT QAiModelPluginFactory : public QObject
{
    Q_OBJECT
public:
    QAiModelPluginFactory() {}

    virtual AiModelPrivateInterface* createInterface() = 0;

};
#define QAiModelPluginFactory_iid "org.qt-project.Qt.QAiModelFactoryInterface"
Q_DECLARE_INTERFACE(QAiModelPluginFactory, QAiModelPluginFactory_iid)

#endif // QAIMODELINTERFACE_P_H
