#!/usr/bin/python3

# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

from http.server import BaseHTTPRequestHandler,HTTPServer
from os import curdir, sep
import re
import cgi
import torch
import simplejson
import base64
from io import StringIO
from io import BytesIO
from PIL import Image
import numpy as np

from ultralytics import YOLO


PORT_NUMBER = 8004

#This class will handles any incoming request from
#the browser 
class myHandler(BaseHTTPRequestHandler):

    #Handler for the POST requests
    def do_POST(self):
        print("do_POST");
        if self.path=="/send":
            self.data_string = self.rfile.read(int(self.headers['Content-Length']))
            print("data_string: " + self.data_string.decode())
            json_data = simplejson.loads(self.data_string)
            #print("json_data: " + simplejson.dumps(json_data))
            print("MODEL: " + json_data["model"])

            # Perform object detection
            base64_image_data = json_data["frame"]
            frameInB64 = base64.b64decode(base64_image_data)
            print("Frame length:" + str(len(frameInB64)))
            frame = Image.open(BytesIO(frameInB64))
            results = model(frame)

            # Render results on the frame
            #frame = results[0].plot()
            print(results[0].to_json())

            print("Sending response")
            self.send_response(200)
            self.end_headers()

            json_response_text = results[0].to_json()
            self.wfile.write(json_response_text.encode("utf-8"))
            return


try:
    #Create a web server and define the handler to manage the
    #incoming request
    server = HTTPServer(('', PORT_NUMBER), myHandler)
    print('Started httpserver on port ' , PORT_NUMBER)

    # Load YOLOv9 model
    model = YOLO('yolov9c.pt')  

    #Wait forever for incoming htto requests
    server.serve_forever()

except KeyboardInterrupt:
    print('^C received, shutting down the web server')
    server.socket.close()


