// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

// TODO: check if all is really needed here
import QtQuick 2.15
import QtQuick.Layouts
import QtQuick.Controls
import Qt.labs.platform // TODO: Is this used here?
import QtMultimedia

import qtaimodel

Rectangle {
    id: rectangle
    anchors.fill: parent
    color: "#ffffff"  // TODO: Use some better color?

    property string llamaPrompt: "You are an assistant.\n"
    property string imageFile: ""

    ColumnLayout {
        RowLayout {
            ColumnLayout {
                RowLayout {
                    Rectangle {
                        width: 20
                        height: 20
                        color: speechToText.processing ? "red" : "green"
                    }

                    Button {
                        visible: recorder.recorderState !== MediaRecorder.RecordingState
                        text: "Record audio"
                        onClicked: {
                            recorder.record()
                        }
                    }
                    Button {
                        visible: recorder.recorderState === MediaRecorder.RecordingState
                        text: "Stop audio recording"
                        onClicked: {
                            recorder.stop()
                            if (recorder.actualLocation != "") {
                                speechToText.pushData(recorder.actualLocation)
                            }
                            if (imageFile != "") {
                                imageToText.pushData(imageFile)
                            }
                        }
                    }
                    CaptureSession {
                        audioInput: AudioInput {}
                        recorder: MediaRecorder {
                            id: recorder
                            mediaFormat {
                                fileFormat: MediaFormat.Wave
                            }
                        }
                    }

                    MultiModal {
                        id: speechToText
                        type: (MultiModal.InputAudio | MultiModal.OutputText)
                        model: "turbo"
                    }
                }

                RowLayout {
                    Rectangle {
                        width: 20
                        height: 20
                        color: imageToText.processing ? "red" : "green"
                    }

                    Button {
                        text: qsTr("Open image")
                        onClicked: fileDialog.open()


                        FileDialog {
                            id: fileDialog
                            folder: StandardPaths.standardLocations(StandardPaths.PicturesLocation)[0]
                            nameFilters: ["*.*"]
                            onAccepted: {
                                imageFile = fileDialog.file
                            }
                            onRejected: {}
                        }
                    }
                    Text {
                        id: result
                        text: rectangle.imageFile
                    }
                    MultiModal {
                        id: imageToText
                        type: (MultiModal.InputImage | MultiModal.OutputText)
                        model: "llava-phi3"  // TODO: replace with Janus model from DeepSeek
                        prompt: "What is in the picture?"
                        optional: true
                        buffered: true
                    }
                }

                RowLayout {
                    Rectangle {
                        width: 20
                        height: 20
                        color: diffuser.processing ? "red" : "green"
                    }
                    TextField {
                        placeholderText: "Text2Image"
                        implicitWidth: 300
                        onEditingFinished: diffuser.pushData(text)
                    }
                    MultiModal {
                        id: diffuser
                        type: (MultiModal.InputText | MultiModal.OutputImage)
                        model: 'IDKiro/sdxs-512-dreamshaper'
                    }

                    Connections {
                        target: diffuser
                        function onGotResult(result) {
                            imageLocation.source = result
                        }
                    }
                }


                TextField {
                    text: llamaPrompt
                    placeholderText: "Llama prompt"
                    implicitWidth: 300
                    onEditingFinished: llamaModel.prompt = text
                }
                RowLayout {
                    Rectangle {
                        width: 20
                        height: 20
                        color: llamaModel.processing ? "red" : "green"
                    }
                    TextField {
                        placeholderText: "Text2Text"
                        implicitWidth: 300
                        onEditingFinished: llamaModel.pushData(text)
                    }
                }

                TextArea {
                    placeholderText: "Enter context"
                    background: Rectangle {
                        color: "lightgreen"
                    }

                    implicitWidth: 300
                    implicitHeight: 100
                    onEditingFinished: llamaModel.documents = [text]
                }
                MultiModal {
                    id: llamaModel
                    type: (MultiModal.InputText | MultiModal.OutputText)
                    model: "gemma3:4b"
                    prompt: llamaPrompt
                    inputs: [ imageToText, speechToText ]
                }

                RowLayout {
                    Rectangle {
                        width: 20
                        height: 20
                        color: text2speech.processing ? "red" : "green"
                    }
                    TextArea {
                        implicitWidth: 300
                        implicitHeight: 100
                        background: Rectangle {
                            color: "lightblue"
                        }

                    }

                    MultiModal {
                        id: text2speech
                        type: (MultiModal.InputText | MultiModal.OutputAudio)
                        inputs: [ llamaModel ]
                    }
                }

            }

            Rectangle {
                width: 300
                height: 300
                color: "red"
                Image {
                    anchors.fill: parent
                    anchors.margins: 2
                    id: imageLocation
                    source: imageFile
                    fillMode: Image.PreserveAspectFit
                }
            }
        }




    }







}
