// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only

#include <QtCore>
#include <QGuiApplication>
#include <QQmlApplicationEngine>


int main(int argc, char *argv[])
{
    QGuiApplication app(argc, argv);

    QQmlApplicationEngine engine;
    const QUrl url("qrc:///App.qml");
    QObject::connect(
                &engine, &QQmlApplicationEngine::objectCreated, &app,
                [url](QObject *obj, const QUrl &objUrl) {
        if (!obj && url == objUrl)
            QCoreApplication::exit(-1);
    }, Qt::QueuedConnection);

    engine.addImportPath("qml");
    engine.addImportPath("../../qml"); // For QML plugins
    QCoreApplication::addLibraryPath("../../plugins"); // For backend plugins

    engine.loadFromModule("qtaiinferenceapi", "App");

    qDebug() << "Standard path for pictures: " << QStandardPaths::standardLocations(QStandardPaths::PicturesLocation)
             << "QML import path" << engine.importPathList();

    if (engine.rootObjects().isEmpty())
        return -1;

    return app.exec();
}
